#pragma once

#include "IO.h"

namespace APE
{

#pragma pack(push, 1)

class CWholeFileIO : public CIO
{
public:
    // construction / destruction
    CWholeFileIO(CIO * pSource);
    ~CWholeFileIO();

    // open / close
    int Open(const wchar_t * pName, bool bOpenReadOnly = false) APE_OVERRIDE;
    int Close() APE_OVERRIDE;

    // read / write
    int Read(void * pBuffer, unsigned int nBytesToRead, unsigned int * pBytesRead) APE_OVERRIDE;
    int Write(const void * pBuffer, unsigned int nBytesToWrite, unsigned int * pBytesWritten) APE_OVERRIDE;

    // seek
    int64 PerformSeek() APE_OVERRIDE;

    // other functions
    int SetEOF() APE_OVERRIDE;
    unsigned char * GetBuffer(int *) APE_OVERRIDE { return APE_NULL; }

    // creation / destruction
    int Create(const wchar_t * pName) APE_OVERRIDE;
    int Delete() APE_OVERRIDE;

    // attributes
    int64 GetPosition() APE_OVERRIDE;
    int64 GetSize() APE_OVERRIDE;
    int GetName(wchar_t * pBuffer) APE_OVERRIDE;

private:

    CSmartPtr<CIO> m_spSource;
    CSmartPtr<unsigned char> m_spWholeFile;
    CSmartPtr<unsigned char> m_spBuffer;
    int m_nOpenResult;
    int64 m_nWholeFilePointer;
    int64 m_nWholeFileSize;
};

#pragma pack(pop)

}
